@@ -12,8 +12,20 @@ import sys
 from zipfile import ZipFile
 from zipfile import BadZipFile
 
-# The VBA project file we want to extract.
+
+def extract_file(xlsm_zip, filename):
+    # Extract a single file from an Excel xlsm macro file.
+    data = xlsm_zip.read("xl/" + filename)
+
+    # Write the data to a local file.
+    file = open(filename, "wb")
+    file.write(data)
+    file.close()
+
+
+# The VBA project file and project signature file we want to extract.
 vba_filename = "vbaProject.bin"
+vba_signature_filename = "vbaProjectSignature.bin"
 
 # Get the xlsm file name from the commandline.
 if len(sys.argv) > 1:
@@ -21,7 +33,9 @@ if len(sys.argv) > 1:
 else:
     print(
         "\nUtility to extract a vbaProject.bin binary from an Excel 2007+ "
-        "xlsm macro file for insertion into an XlsxWriter file."
+        "xlsm macro file for insertion into an XlsxWriter file.\n"
+        "If the macros are digitally signed, extracts also a vbaProjectSignature.bin "
+        "file.\n"
         "\n"
         "See: https://xlsxwriter.readthedocs.io/working_with_macros.html\n"
         "\n"
@@ -34,12 +48,13 @@ try:
     xlsm_zip = ZipFile(xlsm_file, "r")
 
     # Read the xl/vbaProject.bin file.
-    vba_data = xlsm_zip.read("xl/" + vba_filename)
+    extract_file(xlsm_zip, vba_filename)
+    print("Extracted: %s" % vba_filename)
+
+    if "xl/" + vba_signature_filename in xlsm_zip.namelist():
+        extract_file(xlsm_zip, vba_signature_filename)
+        print("Extracted: %s" % vba_signature_filename)
 
-    # Write the vba data to a local file.
-    vba_file = open(vba_filename, "wb")
-    vba_file.write(vba_data)
-    vba_file.close()
 
 except IOError as e:
     print("File error: %s" % str(e))
@@ -61,5 +76,3 @@ except Exception as e:
     # Catch any other exceptions.
     print("File error: %s" % str(e))
     exit()
-
-print("Extracted: %s" % vba_filename)
