@@ -63,7 +63,7 @@ class Packager(object):
         | | |____ theme1.xml
         | |
         | |_____rels
-        | |____ workbook.xml.rels
+        |   |____ workbook.xml.rels
         |
         |_____rels
           |____ .rels
@@ -151,6 +151,8 @@ class Packager(object):
         self._write_drawing_rels_files()
         self._add_image_files()
         self._add_vba_project()
+        self._add_vba_project_signature()
+        self._write_vba_project_rels_file()
         self._write_core_file()
         self._write_app_file()
         self._write_metadata_file()
@@ -304,17 +306,19 @@ class Packager(object):
         properties = self.workbook.doc_properties
         app = App()
 
-        # Add the Worksheet heading pairs.
-        app._add_heading_pair(["Worksheets", self.worksheet_count])
-
-        # Add the Chartsheet heading pairs.
-        app._add_heading_pair(["Charts", self.chartsheet_count])
-
         # Add the Worksheet parts.
+        worksheet_count = 0
         for worksheet in self.workbook.worksheets():
             if worksheet.is_chartsheet:
                 continue
-            app._add_part_name(worksheet.name)
+
+            # Don't write/count veryHidden sheets.
+            if worksheet.hidden != 2:
+                app._add_part_name(worksheet.name)
+                worksheet_count += 1
+
+        # Add the Worksheet heading pairs.
+        app._add_heading_pair(["Worksheets", worksheet_count])
 
         # Add the Chartsheet parts.
         for worksheet in self.workbook.worksheets():
@@ -322,6 +326,9 @@ class Packager(object):
                 continue
             app._add_part_name(worksheet.name)
 
+        # Add the Chartsheet heading pairs.
+        app._add_heading_pair(["Charts", self.chartsheet_count])
+
         # Add the Named Range heading pairs.
         if self.named_ranges:
             app._add_heading_pair(["Named Ranges", len(self.named_ranges)])
@@ -402,9 +409,11 @@ class Packager(object):
         if self.workbook.str_table.count:
             content._add_shared_strings()
 
-        # Add vbaProject if present.
+        # Add vbaProject (and optionally vbaProjectSignature) if present.
         if self.workbook.vba_project:
             content._add_vba_project()
+            if self.workbook.vba_project_signature:
+                content._add_vba_project_signature()
 
         # Add the custom properties if present.
         if self.workbook.custom_properties:
@@ -475,7 +484,7 @@ class Packager(object):
     def _get_table_count(self):
         # Count the table files. Required for the [Content_Types] file.
         for worksheet in self.workbook.worksheets():
-            for table_props in worksheet.tables:
+            for _ in worksheet.tables:
                 self.table_count += 1
 
     def _write_root_rels_file(self):
@@ -557,7 +566,7 @@ class Packager(object):
             rels = Relationships()
 
             for link_data in external_links:
-                rels._add_worksheet_relationship(*link_data)
+                rels._add_document_relationship(*link_data)
 
             # Create .rels file such as /xl/worksheets/_rels/sheet1.xml.rels.
             rels._set_xml_writer(
@@ -574,7 +583,9 @@ class Packager(object):
 
             index += 1
 
-            external_links = worksheet.external_drawing_links
+            external_links = (
+                worksheet.external_drawing_links + worksheet.external_vml_links
+            )
 
             if not external_links:
                 continue
@@ -583,7 +594,7 @@ class Packager(object):
             rels = Relationships()
 
             for link_data in external_links:
-                rels._add_worksheet_relationship(*link_data)
+                rels._add_document_relationship(*link_data)
 
             # Create .rels file such as /xl/chartsheets/_rels/sheet1.xml.rels.
             rels._set_xml_writer(
@@ -629,6 +640,23 @@ class Packager(object):
         )
         rels._assemble_xml_file()
 
+    def _write_vba_project_rels_file(self):
+        # Write the vbaProject.rels xml file if signed macros exist.
+        vba_project_signature = self.workbook.vba_project_signature
+
+        if not vba_project_signature:
+            return
+
+        # Create the vbaProject .rels dir.
+        rels = Relationships()
+
+        rels._add_ms_package_relationship(
+            "/vbaProjectSignature", "vbaProjectSignature.bin"
+        )
+
+        rels._set_xml_writer(self._filename("xl/_rels/vbaProject.bin.rels"))
+        rels._assemble_xml_file()
+
     def _add_image_files(self):
         # Write the /xl/media/image?.xml files.
         workbook = self.workbook
@@ -675,10 +703,45 @@ class Packager(object):
 
             index += 1
 
+    def _add_vba_project_signature(self):
+        # Copy in a vbaProjectSignature.bin file.
+        vba_project_signature = self.workbook.vba_project_signature
+        vba_project_signature_is_stream = self.workbook.vba_project_signature_is_stream
+
+        if not vba_project_signature:
+            return
+
+        xml_vba_signature_name = "xl/vbaProjectSignature.bin"
+
+        if not self.in_memory:
+            # In file mode we just write or copy the VBA project signature file.
+            os_filename = self._filename(xml_vba_signature_name)
+
+            if vba_project_signature_is_stream:
+                # The data is in a byte stream. Write it to the target.
+                os_file = open(os_filename, mode="wb")
+                os_file.write(vba_project_signature.getvalue())
+                os_file.close()
+            else:
+                copy(vba_project_signature, os_filename)
+
+        else:
+            # For in-memory mode we read the vba into a stream.
+            if vba_project_signature_is_stream:
+                # The data is already in a byte stream.
+                os_filename = vba_project_signature
+            else:
+                vba_file = open(vba_project_signature, mode="rb")
+                vba_data = vba_file.read()
+                os_filename = BytesIO(vba_data)
+                vba_file.close()
+
+            self.filenames.append((os_filename, xml_vba_signature_name, True))
+
     def _add_vba_project(self):
         # Copy in a vbaProject.bin file.
         vba_project = self.workbook.vba_project
-        vba_is_stream = self.workbook.vba_is_stream
+        vba_project_is_stream = self.workbook.vba_project_is_stream
 
         if not vba_project:
             return
@@ -689,7 +752,7 @@ class Packager(object):
             # In file mode we just write or copy the VBA file.
             os_filename = self._filename(xml_vba_name)
 
-            if vba_is_stream:
+            if vba_project_is_stream:
                 # The data is in a byte stream. Write it to the target.
                 os_file = open(os_filename, mode="wb")
                 os_file.write(vba_project.getvalue())
@@ -699,7 +762,7 @@ class Packager(object):
 
         else:
             # For in-memory mode we read the vba into a stream.
-            if vba_is_stream:
+            if vba_project_is_stream:
                 # The data is already in a byte stream.
                 os_filename = vba_project
             else:
